@@ -35,7 +35,32 @@ struct _GMountSource
   char *obj_path;
 };
 
+/* We use this hack to avoid problems when this code
+   is shared in both the daemon and the client */
+static GType _g_mount_source_get_type (void) G_GNUC_CONST;
+
+#define g_mount_source_get_type _g_mount_source_get_type
 G_DEFINE_TYPE (GMountSource, g_mount_source, G_TYPE_OBJECT)
+#undef g_mount_source_get_type
+
+GType
+g_mount_source_get_type (void)
+{
+  static volatile gsize type_volatile = 0;
+  
+  if (g_once_init_enter (&type_volatile))
+    {
+      GType type;
+      
+      type = g_type_from_name ("GMountSource");
+      if (type == 0)
+	type = _g_mount_source_get_type ();
+      
+      g_once_init_leave (&type_volatile, type);
+    }
+  
+  return type_volatile;
+}
 
 static void
 g_mount_source_finalize (GObject *object)
@@ -129,6 +154,7 @@ struct AskPasswordData {
   char          *username;
   char          *domain;
   GPasswordSave  password_save;
+  gboolean	 anonymous;
 };
 
 typedef struct AskSyncData AskSyncData;
@@ -193,10 +219,14 @@ ask_password_reply (DBusMessage *reply,
 	{
 	  data->aborted = aborted;
 
-	  data->password = g_strdup (password);
-	  data->username = g_strdup (username);
-	  data->domain = g_strdup (domain);
+	  if (!anonymous)
+	    {
+	      data->password = g_strdup (password);
+	      data->username = *username == 0 ? NULL : g_strdup (username);
+	      data->domain = *domain == 0 ? NULL : g_strdup (domain);
+	    }
 	  data->password_save = (GPasswordSave)password_save;
+	  data->anonymous = anonymous;
 
 	  /* TODO: handle more args */
 	}
@@ -204,7 +234,7 @@ ask_password_reply (DBusMessage *reply,
 
   if (handled == FALSE)
     {
-      g_simple_async_result_set_error (result, G_IO_ERROR, G_IO_ERROR_FAILED, NULL);
+      g_simple_async_result_set_error (result, G_IO_ERROR, G_IO_ERROR_FAILED, "Internal Error");
     }
 
   g_simple_async_result_complete (result);
@@ -230,7 +260,8 @@ g_mount_source_ask_password_async (GMountSource              *source,
       g_simple_async_report_error_in_idle (G_OBJECT (source),
 					   callback,
 					   user_data,
-					   G_IO_ERROR, G_IO_ERROR_FAILED, NULL); 
+					   G_IO_ERROR, G_IO_ERROR_FAILED, 
+					   "Internal Error"); 
       return;
     }
 
@@ -264,6 +295,28 @@ g_mount_source_ask_password_async (GMountSource              *source,
 
 }
 
+/**
+ * g_mount_source_ask_password_finish:
+ * @source: the source to query
+ * @result: the async result
+ * @aborted: set to %TRUE if the password dialog was aborted by the user
+ * @password_out: the to the password set by the user or to %NULL if none
+ * @user_out: set to the username set by the user or to %NULL if none
+ * @domain_out: set to the domain set by the user or to %NULL if none
+ * @anonymous_out: set to %TRUE if the user selected anonymous login. This
+ *                 should only happen if G_ASK_PASSWORD_ANONYMOUS_SUPPORTED
+ *                 was supplied whe querying the password.
+ * @password_save_out: set to the save flags to use when saving the password
+ *                     in the keyring.
+ *
+ * Requests the reply parameters from a g_mount_source_ask_password_async() 
+ * request. All out parameters can be set to %NULL to ignore them.
+ * <note><para>Please be aware that out parameters other than the password
+ * are set to %NULL if the user don't specify them so make sure to
+ * check them.</para></note>
+ *
+ * Returns: %FALSE if the async reply contained an error.
+ **/
 gboolean
 g_mount_source_ask_password_finish (GMountSource  *source,
                                     GAsyncResult  *result,
@@ -271,17 +324,18 @@ g_mount_source_ask_password_finish (GMountSource  *source,
                                     char         **password_out,
                                     char         **user_out,
                                     char         **domain_out,
+				    gboolean	  *anonymous_out,
 				    GPasswordSave *password_save_out)
 {
-  AskPasswordData *data;
+  AskPasswordData *data, def = { TRUE, };
   GSimpleAsyncResult *simple;
 
   simple = G_SIMPLE_ASYNC_RESULT (result);
 
   if (g_simple_async_result_propagate_error (simple, NULL))
-    return FALSE;
-
-  data = (AskPasswordData *) g_simple_async_result_get_op_res_gpointer (simple);
+    data = &def;
+  else
+    data = (AskPasswordData *) g_simple_async_result_get_op_res_gpointer (simple);
 
   if (aborted)
     *aborted = data->aborted;
@@ -304,10 +358,13 @@ g_mount_source_ask_password_finish (GMountSource  *source,
       data->domain = NULL;
     }
 
+  if (anonymous_out)
+    *anonymous_out = data->anonymous;
+
   if (password_save_out)
     *password_save_out = data->password_save;  
   
-  return TRUE;
+  return data != &def;
 }
 
 
@@ -338,20 +395,12 @@ g_mount_source_ask_password (GMountSource *source,
 			     char **password_out,
 			     char **user_out,
 			     char **domain_out,
+			     gboolean *anonymous_out,
 			     GPasswordSave *password_save_out)
 {
-  char *password, *username, *domain;
-  GPasswordSave password_save;
-  gboolean handled, aborted;
+  gboolean handled;
   AskSyncData data = {NULL};
   
-  if (password_out)
-    *password_out = NULL;
-  if (user_out)
-    *user_out = NULL;
-  if (domain_out)
-    *domain_out = NULL;
-  
   data.mutex = g_mutex_new ();
   data.cond = g_cond_new ();
 
@@ -375,34 +424,14 @@ g_mount_source_ask_password (GMountSource *source,
 
   handled = g_mount_source_ask_password_finish (source,
                                                 data.result,
-                                                &aborted,
-                                                &password,
-                                                &username,
-                                                &domain,
-						&password_save);
+                                                aborted_out,
+                                                password_out,
+                                                user_out,
+                                                domain_out,
+						anonymous_out,
+						password_save_out);
   g_object_unref (data.result);
 
-  if (aborted_out)
-    *aborted_out = aborted;
-
-  if (password_out)
-    *password_out = password;
-  else
-    g_free (password);
-
-  if (user_out)
-    *user_out = username;
-  else
-    g_free (username);
-
-  if (domain_out)
-    *domain_out = domain;
-  else
-    g_free (domain);
-
-  if (password_save_out)
-    *password_save_out = password_save;
-  
   return handled;
 }
 
@@ -432,6 +461,7 @@ op_ask_password_reply (GObject *source_object,
                                                 &username,
                                                 &password,
                                                 &domain,
+						NULL,
 						&password_save);
 
   if (!handled)
@@ -522,7 +552,7 @@ ask_question_reply (DBusMessage *reply,
 
   if (handled == FALSE)
     {
-      g_simple_async_result_set_error (result, G_IO_ERROR, G_IO_ERROR_FAILED, NULL);
+      g_simple_async_result_set_error (result, G_IO_ERROR, G_IO_ERROR_FAILED, "Internal Error");
     }
 
   g_simple_async_result_complete (result);
@@ -591,7 +621,8 @@ g_mount_source_ask_question_async (GMountSource       *source,
       g_simple_async_report_error_in_idle (G_OBJECT (source),
 					   callback,
 					   user_data,
-					   G_IO_ERROR, G_IO_ERROR_FAILED, NULL); 
+					   G_IO_ERROR, G_IO_ERROR_FAILED, 
+					   "Internal Error"); 
       return;
     }
 
@@ -624,15 +655,15 @@ g_mount_source_ask_question_finish (GMountSource *source,
 				    gboolean     *aborted,
 				    gint         *choice_out)
 {
-  AskQuestionData *data;
+  AskQuestionData *data, def= { FALSE, };
   GSimpleAsyncResult *simple;
 
   simple = G_SIMPLE_ASYNC_RESULT (result);
 
   if (g_simple_async_result_propagate_error (simple, NULL))
-    return FALSE;
-
-  data = (AskQuestionData *) g_simple_async_result_get_op_res_gpointer (simple);
+    data = &def;
+  else
+    data = (AskQuestionData *) g_simple_async_result_get_op_res_gpointer (simple);
 
   if (aborted)
     *aborted = data->aborted;
@@ -640,7 +671,7 @@ g_mount_source_ask_question_finish (GMountSource *source,
   if (choice_out)
     *choice_out = data->choice;
 
-  return TRUE;	
+  return data != &def;	
 }
 
 static void
